@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2023 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -29,7 +29,6 @@
    include/config.php for user configurable settings.
 
 */
-
 /* load cacti version from file */
 $cacti_version_file = dirname(__FILE__) . '/cacti_version';
 
@@ -156,7 +155,6 @@ $no_http_header_files = array(
 	'add_graphs.php',
 	'add_perms.php',
 	'add_tree.php',
-	'boost_rrdupdate.php',
 	'cmd.php',
 	'cmd_realtime.php',
 	'copy_user.php',
@@ -237,6 +235,7 @@ if ($config['cacti_server_os'] == 'win32') {
 	$config['base_path']    = preg_replace("/(.*)[\/]include/", "\\1", dirname(__FILE__));
 	$config['library_path'] = preg_replace("/(.*[\/])include/", "\\1lib", dirname(__FILE__));
 }
+
 $config['include_path'] = dirname(__FILE__);
 $config['rra_path'] = $config['base_path'] . '/rra';
 
@@ -286,6 +285,11 @@ $filename = get_current_page();
 $config['is_web'] = !defined('CACTI_CLI_ONLY');
 if ((isset($no_http_headers) && $no_http_headers == true) || in_array($filename, $no_http_header_files, true)) {
 	$config['is_web'] = false;
+
+	if (isset($_REQUEST) && cacti_sizeof($_REQUEST) || !isset($_SERVER['argv'])) {
+		print 'FATAL: This file can only be called from the command line.' . PHP_EOL;
+		exit;
+	}
 }
 
 $auto_start = ini_get('session.auto_start');
@@ -329,9 +333,7 @@ if ($config['poller_id'] > 1 || isset($rdatabase_hostname)) {
 		$remote_db_cnn_id = db_connect_real($rdatabase_hostname, $rdatabase_username, $rdatabase_password, $rdatabase_default, $rdatabase_type, $rdatabase_port, $database_retries, $rdatabase_ssl, $rdatabase_ssl_key, $rdatabase_ssl_cert, $rdatabase_ssl_ca);
 	}
 
-	if ($config['is_web'] && is_object($remote_db_cnn_id) &&
-		$config['connection'] != 'recovery' &&
-		$config['cacti_db_version'] != 'new_install') {
+	if ($config['is_web'] && is_object($remote_db_cnn_id) && $config['connection'] != 'recovery' && $config['cacti_db_version'] != 'new_install' && !defined('IN_CACTI_INSTALL')) {
 
 		// Connection worked, so now override the default settings so that it will always utilize the remote connection
 		$database_default   = $rdatabase_default;
