@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2023 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -172,7 +172,7 @@ $fields_snmp_item_with_oids = $fields_snmp_item + array(
 		'friendly_name' => __('Bulk Walk Maximum Repetitions'),
 		'description'   => __('For SNMPv2 and SNMPv3 Devices, the SNMP Bulk Walk max-repetitions size. The default is to \'Auto Detect on Re-Index\'. For very large switches, high performance servers, Jumbo Frame Networks or for high latency WAN connections, increasing this value may increase poller performance. More data is packed into a single SNMP packet which can reduce data query run time. However, some devices may completely refuse to respond to packets with a max-repetition size which is set too large. This can be especially true for lower-powered IoT type devices or smaller embedded IT appliances. Special attention to the overall network path MTU should also be considered since setting a value which is too high could lead to packet fragmentation.'),
 		'value'         => '|arg1:bulk_walk_size|',
-		'default'       => '-1',
+		'default'       => '0',
 		'array' => array(
 			-1 => __('Auto Detect on Re-Index'),
 			0  => __('Auto Detect/Set on first Re-Index'),
