@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2023 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -54,9 +54,9 @@ function run_data_query($host_id, $snmp_query_id, $automation = false, $force =
 			WHERE id = ?',
 			array($poller_id));
 
-		$fgc_contextoption = get_default_contextoption();
-		$fgc_context       = stream_context_create($fgc_contextoption);
-		$response          = @file_get_contents(get_url_type() . '://' . $hostname . $config['url_path'] . '/remote_agent.php?action=runquery&host_id=' . $host_id . '&data_query_id=' . $snmp_query_id, false, $fgc_context);
+		$url = $config['url_path'] . 'remote_agent.php?action=runquery&host_id=' . $host_id . '&data_query_id=' . $snmp_query_id;
+
+		$response = call_remote_data_collector($poller_id, $url);
 
 		if ($response != '') {
 			$response = json_decode($response, true);
@@ -1081,7 +1081,11 @@ function query_snmp_host($host_id, $snmp_query_id) {
 						$ip_value = '';
 
 						foreach($parts as $idx => $part) {
-							$parts[$idx] = substr(strtoupper('00' . dechex($part)), -2);
+							if (is_numeric($part)) {
+								$parts[$idx] = substr(strtoupper('00' . dechex($part)), -2);
+							} else {
+								$parts[$idx] = substr(strtoupper('00' . $part), -2);
+							}
 
 							if ($idx % 2 == 0 && $idx > 0) {
 								$ip_value .= ':' . $parts[$idx];
@@ -1375,7 +1379,11 @@ function query_snmp_host($host_id, $snmp_query_id) {
 						// Check for an IPv6 Address or Hex String
 						$parts = explode('.', $parse_value);
 						foreach($parts as $idx => $part) {
-							$parts[$idx] = substr(strtoupper('00' . dechex($part)), -2);
+							if (is_numeric($part)) {
+								$parts[$idx] = substr(strtoupper('00' . dechex($part)), -2);
+							} else {
+								$parts[$idx] = substr(strtoupper('00' . $part), -2);
+							}
 
 							if ($idx % 2 == 0 && $idx > 0) {
 								$ip_value .= ':' . $parts[$idx];
@@ -2358,10 +2366,10 @@ function get_script_query_path($args, $script_path, $host_id) {
 /**
  * verify a given index_order
  * @param array $raw_xml 	- parsed XML array
+ *
  * @return bool 			- index_order field valid
  */
 function verify_index_order($raw_xml) {
-
 	/* invalid xml check */
 	if ((!is_array($raw_xml)) || (cacti_sizeof($raw_xml) == 0)) {
 		query_debug_timer_offset('data_query', __esc('Error parsing XML file into an array.'));
@@ -2372,13 +2380,16 @@ function verify_index_order($raw_xml) {
 
 	/* list each of the input fields for this snmp query */
 	foreach ($raw_xml['fields'] as $field_name => $field_array) {
-		if ($field_array['direction'] == 'input' || $field_array['direction'] == 'input-output') {
-			/* create a list of all values for this index */
-			array_push($xml_inputs, $field_name);
+		if (is_array($field_array) && isset($field_array['direction'])) {
+			if ($field_array['direction'] == 'input' || $field_array['direction'] == 'input-output') {
+				/* create a list of all values for this index */
+				array_push($xml_inputs, $field_name);
+			}
 		}
 	}
 
 	$all_index_order_fields_found = true;
+
 	/* the xml file contains an ordered list of 'indexable' fields */
 	if (isset($raw_xml['index_order'])) {
 		$index_order_array = explode(':', $raw_xml['index_order']);
