@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2023 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -227,10 +227,25 @@ function db_connect_real($device, $user, $pass, $db_name, $db_type = 'mysql', $p
 	return false;
 }
 
+/**
+ * db_check_reconnect - Check the database connection.  If the connection is gone
+ *  attempt to reconnect, otherwise return the connection
+ *
+ * @param bool|object  The connection to check
+ * @param bool         Wether or not to log the connection check
+ *
+ * @return bool        The database true is the database is connected else false
+ */
 function db_check_reconnect($db_conn = false, $log = true) {
 	global $config, $database_details;
 
-	include($config['base_path'] . '/include/config.php');
+	if (file_exists($config['base_path'] . '/include/config.php')) {
+		include($config['base_path'] . '/include/config.php');
+	} else {
+		global $database_hostname, $database_username, $database_password, $database_default;
+		global $database_type, $database_port, $database_retries;
+		global $database_ssl, $database_ssl_key, $database_ssl_cert, $database_ssl_ca;
+	}
 
 	if (cacti_sizeof($database_details) && $db_conn !== false) {
 		foreach($database_details as $det) {
@@ -273,7 +288,7 @@ function db_check_reconnect($db_conn = false, $log = true) {
 		db_close();
 
 		// Connect to the database server
-		db_connect_real(
+		$cnn_id = db_connect_real(
 			$database_hostname,
 			$database_username,
 			$database_password,
@@ -286,6 +301,14 @@ function db_check_reconnect($db_conn = false, $log = true) {
 			$database_ssl_cert,
 			$database_ssl_ca
 		);
+
+		if ($cnn_id !== false) {
+			return true;
+		} else {
+			return false;
+		}
+	} else {
+		return true;
 	}
 }
 
