@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -73,13 +73,28 @@ function api_plugin_hook($name) {
 
 	if (!empty($result)) {
 		foreach ($result as $hdata) {
-			if (!in_array($hdata['name'], $plugins_integrated)) {
-				if (file_exists($config['base_path'] . '/plugins/' . $hdata['name'] . '/' . $hdata['file'])) {
-					include_once($config['base_path'] . '/plugins/' . $hdata['name'] . '/' . $hdata['file']);
+			// Security check
+			if (strpos($hdata['file'], '..') !== false) {
+				cacti_log("ERROR: Attempted inclusion of not plugin file $plugin_file from $plugin_name with the hook name $name", false, 'SECURITY');
+				continue;
+			}
+
+			$plugin_name = $hdata['name'];
+
+			if (!in_array($plugin_name, $plugins_integrated, true)) {
+				$plugin_func = $hdata['function'];
+				$plugin_file = $hdata['file'];
+				$full_path   = $config['base_path'] . '/plugins/' . $plugin_name . '/' . $plugin_file;
+				$debounce    = 'mpf_' . $plugin_name . '_' . $plugin_func;
+
+				if (file_exists($full_path)) {
+					include_once($full_path);
 				}
-				$function = $hdata['function'];
-				if (function_exists($function)) {
-					api_plugin_run_plugin_hook($name, $hdata['name'], $function, $args);
+
+				if (function_exists($plugin_func)) {
+					api_plugin_run_plugin_hook($name, $plugin_name, $plugin_func, $args);
+				} else if (debounce_run_notification($debounce)) {
+					cacti_log(sprintf('WARNING: Function "%s" does not exist in %s/%s for hook "%s"' . PHP_EOL, $plugin_func, $plugin_name, $plugin_file, $name), false, 'PLUGIN', POLLER_VERBOSITY_MEDIUM);
 				}
 			}
 		}
@@ -147,7 +162,7 @@ function api_plugin_hook_function($name, $parm = NULL) {
 
 					if (($is_array && !is_array($ret)) || ($ret == null && $null_ret === false)) {
 						if (cacti_sizeof($result) > 1) {
-							cacti_log(sprintf("WARNING: Plugin hook '%s' from Plugin '%s' must return the calling array or variable, and it is not doing so.  Please report this to the Plugin author.", $function, $hdata['name']), false);
+							cacti_log(sprintf("WARNING: Plugin hook '%s' from Plugin '%s' must return the calling array or variable, and it is not doing so.  Please report this to the Plugin author.", $function, $hdata['name']), false, 'PLUGIN');
 						}
 					}
 				}
@@ -169,14 +184,12 @@ function api_plugin_run_plugin_hook($hook, $plugin, $function, $args) {
 
 		$required_capabilities = array(
 			// Poller related
-			'poller_top'               => array('remote_collect'), // Poller Top
-			'poller_bottom'            => array('remote_collect'), // Poller execution, api_plugin_hook
-			'update_host_status'       => array('remote_collect'), // Processing poller output, api_plugin_hook
-			'poller_output'            => array('remote_collect'), // Poller output activities
-			'poller_command_args'      => array('remote_collect'), // Command line arguments
-			'cacti_stats_update'       => array('remote_collect'), // Updating statistics
-			'poller_finishing'         => array('remote_collect'), // Poller post processing
-			'poller_exiting'           => array('remote_collect'), // Poller exception handling
+			'poller_top'               => array('remote_collect'),              // Poller Top, api_plugin_hook
+			'poller_bottom'            => array('remote_poller'),               // Poller execution, api_plugin_hook
+			'update_host_status'       => array('remote_collect'),              // Processing poller output, api_plugin_hook
+			'poller_output'            => array('remote_collect'),              // Poller output activities
+			'poller_finishing'         => array('remote_collect'),              // Poller post processing, api_plugin_hook
+			'poller_exiting'           => array('remote_collect'),              // Poller exception handling, api_plugin_hook
 
 			// GUI Related
 			'page_head'                => array('online_view', 'offline_view'), // Navigation, api_plugin_hook
@@ -192,19 +205,28 @@ function api_plugin_run_plugin_hook($hook, $plugin, $function, $args) {
 		if ($plugin_capabilities === false) {
 			$function($args);
 		} elseif (api_plugin_hook_is_remote_collect($hook, $plugin, $required_capabilities)) {
-			if (api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities)) {
+			if (api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities, $plugin)) {
 				$function($args);
 			}
 		} elseif (isset($required_capabilities[$hook])) {
-			if (api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities)) {
+			if (api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities, $plugin)) {
 				$function($args);
 			}
-		} else {
+		} elseif ($config['connection'] == 'online' ||
+			((api_plugin_has_capability($plugin, 'offline_mgmt') || api_plugin_has_capability($plugin, 'offline_view'))
+			&& $config['connection'] != 'online')) {
 			$function($args);
+		} else {
+			// Don't run as they are not required
 		}
 
 		// See if we need to restore the menu to original
-		if (($hook == 'config_arrays' || 'config_insert') && $config['connection'] == 'offline') {
+		$remote_hooks = array(
+			'config_arrays',
+			'config_insert',
+		);
+
+		if (in_array($hook, $remote_hooks) && ($config['connection'] == 'offline' || $config['connection'] == 'recovery')) {
 			if (!api_plugin_has_capability($plugin, 'offline_mgmt')) {
 				if ($orig_menu !== $menu) {
 					$menu = $orig_menu;
@@ -224,7 +246,8 @@ function api_plugin_run_plugin_hook_function($hook, $plugin, $function, $ret) {
 	if ($config['poller_id'] > 1) {
 		$required_capabilities = array(
 			// Poller related
-			'poller_output'            => array('remote_collect'), // Processing poller output, api_plugin_hook_function
+			'poller_output'            => array('remote_collect'),              // Processing poller output, api_plugin_hook_function
+			'cacti_stats_update'       => array('remote_collect'),              // Updating Cacti stats
 
 			// GUI Related
 			'top_header'               => array('online_view', 'offline_view'), // Top Tabs, api_plugin_hook_function
@@ -239,17 +262,17 @@ function api_plugin_run_plugin_hook_function($hook, $plugin, $function, $ret) {
 
 		$plugin_capabilities = api_plugin_remote_capabilities($plugin);
 
-		// we will run if capabilities are not set
 		if ($plugin_capabilities === false) {
+			// we will run if capabilities are not set
 			$ret = $function($ret);
-		// run if hooks is remote_collect and we support it
 		} elseif (api_plugin_hook_is_remote_collect($hook, $plugin, $required_capabilities)) {
-			if (api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities)) {
+			// run if hook is remote_collect and we support it
+			if (api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities, $plugin)) {
 				$ret = $function($ret);
 			}
-		// run if hooks is remote_collect and we support it
 		} elseif (isset($required_capabilities[$hook])) {
-			if (api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities)) {
+			// run if hook is remote_collect and we support it
+			if (api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities, $plugin)) {
 				$ret = $function($ret);
 			}
 		} else {
@@ -372,17 +395,33 @@ function api_plugin_has_capability($plugin, $capability) {
 	}
 }
 
-function api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities) {
+function api_plugin_status_run($hook, $required_capabilities, $plugin_capabilities, $plugin = '') {
 	global $config;
 
 	$status = $config['connection'];
 
+	if ($plugin == '') {
+		cacti_log('WARNING: The function \'api_plugin_status_run\' API has changed.  Please add the $plugin attribute to the last position', false, 'PLUGIN');
+		$plugin = 'Unknown';
+	}
+
+	// Don't run if not a supported hook
 	if (!isset($required_capabilities[$hook])) {
-		return true;
+		cacti_log(sprintf('WARNING: Not running hook %s for plugin %s as its not a supported Remote Hook', $hook, $plugin), false, 'PLUGIN');
+
+		return false;
 	}
 
 	foreach($required_capabilities[$hook] as $capability) {
-		if ($status == 'online' && strpos($capability, 'online') === false) {
+		if ($capability == 'remote_collect') {
+			if (strpos($plugin_capabilities, "$capability:1") !== false) {
+				return true;
+			}
+		} elseif ($capability == 'remote_poller') {
+			if (strpos($plugin_capabilities, "$capability:1") !== false) {
+				return true;
+			}
+		} elseif ($status == 'online' && strpos($capability, 'online') === false) {
 			continue;
 		} elseif (($status == 'offline' || $status == 'recovery') && strpos($capability, 'offline') === false) {
 			continue;
@@ -491,7 +530,11 @@ function api_plugin_db_table_create($plugin, $table, $data) {
 
 		$sql .= ') ENGINE = ' . $data['type'];
 
-		if (isset($data['row_format']) && db_get_global_variable('innodb_file_format') == 'Barracuda') {
+		if (isset($data['charset'])) {
+			$sql .= ' DEFAULT CHARSET = ' . $data['charset'];
+		}
+
+		if (isset($data['row_format']) && strtolower(db_get_global_variable('innodb_file_format')) == 'barracuda') {
 			$sql .= ' ROW_FORMAT = ' . $data['row_format'];
 		}
 
@@ -508,14 +551,16 @@ function api_plugin_db_table_create($plugin, $table, $data) {
 			if (isset($data['collate'])) {
 				db_execute("ALTER TABLE `$table` COLLATE = " . $data['collate']);
 			}
-
-			if (isset($data['charset'])) {
-				db_execute("ALTER TABLE `$table` CHARSET = " . $data['charset']);
-			}
 		}
 	}
 }
 
+function api_plugin_drop_table($table) {
+	db_execute("DROP TABLE IF EXISTS $table");
+
+	api_plugin_drop_remote_table($table);
+}
+
 function api_plugin_db_changes_remove($plugin) {
 	$tables = db_fetch_assoc_prepared("SELECT `table`
 		FROM plugin_db_changes
@@ -528,6 +573,8 @@ function api_plugin_db_changes_remove($plugin) {
 			db_execute('DROP TABLE IF EXISTS `' . $table['table'] . '`;');
 		}
 
+		api_plugin_drop_remote_table($table['table']);
+
 		db_execute_prepared("DELETE FROM plugin_db_changes
 			WHERE plugin = ?
 			AND method ='create'",
@@ -552,7 +599,7 @@ function api_plugin_db_changes_remove($plugin) {
 	}
 }
 
-function api_plugin_db_add_column ($plugin, $table, $column) {
+function api_plugin_db_add_column($plugin, $table, $column) {
 	global $config, $database_default;
 
 	// Example: api_plugin_db_add_column ('thold', 'plugin_config',
@@ -639,6 +686,7 @@ function api_plugin_install($plugin) {
 
 	if (!defined('IN_CACTI_INSTALL')) {
 		define('IN_CACTI_INSTALL', 1);
+		define('IN_PLUGIN_INSTALL', 1);
 	}
 
 	$dependencies = api_plugin_get_dependencies($plugin);
@@ -648,7 +696,7 @@ function api_plugin_install($plugin) {
 	if (!$proceed) {
 		$message .= '<br><br>' . __('Plugin cannot be installed.');
 
-		raise_message($message, MESSAGE_LEVEL_ERROR);
+		raise_message('dependency_check', $message, MESSAGE_LEVEL_ERROR);
 
 		header('Location: plugins.php?header=false');
 
@@ -685,6 +733,12 @@ function api_plugin_install($plugin) {
 
 		$author  = $info['author'];
 		$version = $info['version'];
+	} elseif (strpos($plugin, 'plugin_') !== false) {
+		raise_message('directory_error', __('The Plugin directory \'%s\' needs to be renamed to remove \'plugin_\' from the name before it can be installed.', $plugin), MESSAGE_LEVEL_ERROR);
+		return false;
+	} else {
+		raise_message('version_error', __('The Plugin in the directory \'%s\' does not include an version function \'%s()\'.  This function must exist for the plugin to be installed.', $plugin, $function), MESSAGE_LEVEL_ERROR);
+		return false;
 	}
 
 	db_execute_prepared('INSERT INTO plugin_config
@@ -709,7 +763,12 @@ function api_plugin_install($plugin) {
 				WHERE directory = ?',
 				array($plugin));
 		}
+	} else {
+		raise_message('install_error', __('The Plugin in the directory \'%s\' does not include an install function \'%s()\'.  This function must exist for the plugin to be installed.', $plugin, $function), MESSAGE_LEVEL_ERROR);
+		return false;
 	}
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_uninstall_integrated() {
@@ -748,6 +807,8 @@ function api_plugin_uninstall($plugin, $tables = true) {
 			WHERE plugin = ?',
 			array($plugin));
 	}
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_check_config($plugin) {
@@ -784,15 +845,25 @@ function api_plugin_enable($plugin) {
 }
 
 function api_plugin_is_enabled($plugin) {
+	static $pstatus = array();;
+
+	if (isset($pstatus[$plugin])) {
+		return $pstatus[$plugin];
+	}
+
 	$status = db_fetch_cell_prepared('SELECT status
 		FROM plugin_config
 		WHERE directory = ?',
 		array($plugin), false);
 
 	if ($status == '1') {
+		$pstatus[$plugin] = true;
+
 		return true;
 	}
 
+	$pstatus[$plugin] = false;
+
 	return false;
 }
 
@@ -803,6 +874,62 @@ function api_plugin_disable($plugin) {
 		SET status = 4
 		WHERE directory = ?',
 		array($plugin));
+
+	api_plugin_replicate_config();
+}
+
+function api_plugin_replicate_config() {
+	global $config;
+
+	include_once($config['base_path'] . '/lib/poller.php');
+
+	$gone_time = read_config_option('poller_interval') * 2;
+
+	$pollers = array_rekey(
+		db_fetch_assoc('SELECT
+			id,
+			UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_status) AS last_polled
+			FROM poller
+			WHERE id > 1
+			AND disabled=""'),
+		'id', 'last_polled'
+	);
+
+	if (cacti_sizeof($pollers)) {
+		foreach($pollers as $poller_id => $last_polled) {
+			if ($last_polled < $gone_time) {
+				replicate_out($poller_id, 'plugins');
+			}
+		}
+	}
+}
+
+function api_plugin_drop_remote_table($table) {
+	global $config;
+
+	include_once($config['base_path'] . '/lib/poller.php');
+
+	$gone_time = read_config_option('poller_interval') * 2;
+
+	$pollers = array_rekey(
+		db_fetch_assoc('SELECT
+			id,
+			UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_status) AS last_polled
+			FROM poller
+			WHERE id > 1
+			AND disabled=""'),
+		'id', 'last_polled'
+	);
+
+	if (cacti_sizeof($pollers)) {
+		foreach($pollers as $poller_id => $last_polled) {
+			$rcnn_id = poller_connect_to_remote($poller_id);
+
+			if ($rcnn_id !== false) {
+				db_execute("DROP TABLE IF EXISTS $table", false, $rcnn_id);
+			}
+		}
+	}
 }
 
 function api_plugin_disable_all($plugin) {
@@ -812,6 +939,8 @@ function api_plugin_disable_all($plugin) {
 		SET status = 4
 		WHERE directory = ?',
 		array($plugin));
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_moveup($plugin) {
@@ -833,6 +962,8 @@ function api_plugin_moveup($plugin) {
 		db_execute_prepared('UPDATE plugin_config SET id = ? WHERE id = ?', array($prior_id, $id));
 		db_execute_prepared('UPDATE plugin_config SET id = ? WHERE id = ?', array($id, $temp_id));
 	}
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_movedown($plugin) {
@@ -844,11 +975,17 @@ function api_plugin_movedown($plugin) {
 	db_execute_prepared('UPDATE plugin_config SET id = ? WHERE id = ?', array($temp_id, $next_id));
 	db_execute_prepared('UPDATE plugin_config SET id = ? WHERE id = ?', array($next_id, $id));
 	db_execute_prepared('UPDATE plugin_config SET id = ? WHERE id = ?', array($id, $temp_id));
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_register_hook($plugin, $hook, $function, $file, $enable = false) {
 	$status = 0;
 
+	if (!api_plugin_valid_entrypoint($plugin, __FUNCTION__)) {
+		return false;
+	}
+
 	$exists = db_fetch_cell_prepared('SELECT COUNT(*)
 		FROM plugin_hooks
 		WHERE name = ?
@@ -894,12 +1031,16 @@ function api_plugin_register_hook($plugin, $hook, $function, $file, $enable = fa
 			AND `hook` = ?",
 			array($function, $status, $file, $plugin, $hook));
 	}
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_remove_hooks($plugin) {
 	db_execute_prepared('DELETE FROM plugin_hooks
 		WHERE name = ?',
 		array($plugin));
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_enable_hooks($plugin) {
@@ -907,16 +1048,20 @@ function api_plugin_enable_hooks($plugin) {
 		SET status = 1
 		WHERE name = ?',
 		array($plugin));
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_disable_hooks($plugin) {
 	db_execute_prepared("UPDATE plugin_hooks
-		SET status = 0
+		SET status = 4
 		WHERE name = ?
 		AND hook != 'config_settings'
 		AND hook != 'config_arrays'
 		AND hook != 'config_form'",
 		array($plugin));
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_disable_hooks_all($plugin) {
@@ -924,11 +1069,30 @@ function api_plugin_disable_hooks_all($plugin) {
 		SET status = 0
 		WHERE name = ?",
 		array($plugin));
+
+	api_plugin_replicate_config();
+}
+
+function api_plugin_valid_entrypoint($plugin, $function) {
+	// Check for invalid entrypoint install/upgrade
+	$backtrace = debug_backtrace();
+	if (cacti_sizeof($backtrace)) {
+		if (!preg_match('/(install|upgrade|setup)/i', $backtrace[2]['function'])) {
+			cacti_log(sprintf('WARNING: Plugin \'%s\' is attempting to call \'%s\' improperly in function \'%s\'', $plugin, $function, $backtrace[2]['function']), false, 'PLUGIN');
+			return false;
+		}
+	}
+
+	return true;
 }
 
 function api_plugin_register_realm($plugin, $file, $display, $admin = true) {
 	$files = explode(',', $file);
 
+	if (!api_plugin_valid_entrypoint($plugin, __FUNCTION__)) {
+		return false;
+	}
+
 	$i = 0;
 	$sql_where = '(';
 	foreach($files as $tfile) {
@@ -947,7 +1111,7 @@ function api_plugin_register_realm($plugin, $file, $display, $admin = true) {
 		$realm_id = $realm_ids[0]['id'];
 	} elseif (cacti_sizeof($realm_ids) > 1) {
 		$realm_id = $realm_ids[0]['id'];
-		cacti_log('WARNING: Registering Realm for Plugin ' . $plugin . ' and Filenames ' . $file . ' is ambiguous.  Using first matching Realm.  Contact the plugin owner to resolve this issue.');
+		cacti_log('WARNING: Registering Realm for Plugin ' . $plugin . ' and Filenames ' . $file . ' is ambiguous.  Using first matching Realm.  Contact the plugin owner to resolve this issue.', false, 'PLUGIN');
 
 		unset($realm_ids[0]);
 
@@ -1025,6 +1189,8 @@ function api_plugin_register_realm($plugin, $file, $display, $admin = true) {
 			WHERE id = ?',
 			array($display, $file, $realm_id));
 	}
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_remove_realms($plugin) {
@@ -1047,6 +1213,8 @@ function api_plugin_remove_realms($plugin) {
 	db_execute_prepared('DELETE FROM plugin_realms
 		WHERE plugin = ?',
 		array($plugin));
+
+	api_plugin_replicate_config();
 }
 
 function api_plugin_load_realms() {
